# EllipPy is licensed under The 3-Clause BSD, see LICENSE.
# Copyright 2025 Sira Pornsiriprasert <code@psira.me>

"""
Elliptic integral functions in Bulirsch's form.
"""

from numpy.typing import ArrayLike
from . import _ellip
from ._ellip import FloatArray, returnfloat, returnfloat_single


def cel(kc: ArrayLike, p: ArrayLike, a: ArrayLike, b: ArrayLike) -> FloatArray | float:
    r"""Computes general complete elliptic integral in Bulirsch form ``cel``.

    .. math::

        \mathrm{cel}(k_c, p, a, b) = \int_0^{\pi/2} \frac{a\cos^2\theta + b\sin^2\theta}{(\cos^2\theta + p\sin^2\theta)\,\sqrt{\cos^2\theta + k_c^2\sin^2\theta}}\,\mathrm{d}\theta

    Args:
        kc (ArrayLike): Complementary modulus. kc ∈ ℝ, kc ≠ 0.
        p (ArrayLike): Characteristic parameter. p ∈ ℝ, p ≠ 0.
        a, b (ArrayLike): Real-valued coefficient.

    Returns:
        Scalar or `numpy.ndarray` with the same shape as `kc`.

    Raises:
        ValueError: If kc = 0, p = 0, more than one argument is infinite, or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/cel.html" width="100%" height="500px"></iframe>

    Special Cases:
        - cel(kc, p, 0, 0) = 0
        - cel(kc, p, a, b) = 0 for \|kc\| = ∞
        - cel(kc, p, a, b) = 0 for \|p\| = ∞
        - cel(kc, p, a, b) = sign(a) ∞ for \|a\| = ∞
        - cel(kc, p, a, b) = sign(b) ∞ for \|b\| = ∞

    Related Functions:
        With kc² = 1 - m and p = 1 - n:
            - K(m) = cel(kc, 1, 1, 1) = cel1(kc)
            - E(m) = cel(kc, 1, 1, kc²) = cel2(kc, 1, kc²)
            - D(m) = cel(kc, 1, 0, 1)
            - Π(n, m) = cel(kc, p, 1, 1)

    References:
        - Bulirsch, R. “Numerical Calculation of Elliptic Integrals and Elliptic Functions. III.” Numerische Mathematik 13, no. 4 (August 1, 1969): 305-15. https://doi.org/10.1007/BF02165405.
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.cel, kc, p, a, b)


def cel1(kc: ArrayLike) -> FloatArray | float:
    r"""Computes Bulirsch complete integral of the first kind ``cel1``.

    .. math::

        \mathrm{cel1}(k_c) = \int_0^{\pi/2} \frac{\mathrm{d}\theta}{\sqrt{\cos^2\theta + k_c^2\sin^2\theta}}

    Args:
        kc (ArrayLike): Complementary modulus. kc ∈ ℝ, kc ≠ 0.

    Returns:
        Scalar or `numpy.ndarray` with the same shape as `kc`.

    Raises:
        ValueError: If kc = 0 or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/cel1.html" width="100%" height="500px"></iframe>

    Special Cases:
        - cel1(kc) = 0 for \|kc\| = ∞

    Related Functions:
        - With kc² = 1 - m: K(m) = cel(kc, 1, 1, 1) = cel1(kc)

    References:
        - Bulirsch, Roland. “Numerical Calculation of Elliptic Integrals and Elliptic Functions.” Numerische Mathematik 7, no. 1 (February 1, 1965): 78-90. https://doi.org/10.1007/BF01397975.
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat_single(_ellip.cel1, kc)


def cel2(kc: ArrayLike, a: ArrayLike, b: ArrayLike) -> FloatArray | float:
    r"""Computes Bulirsch complete integral of the second kind ``cel2``.

    .. math::

        \mathrm{cel2}(k_c, a, b) = \int_{0}^{\pi/2}\frac{a + b\,\tan^2\theta}{\sqrt{(1+\tan^2\theta)(1+k_c^2\tan^2\theta)}} \,\mathrm{d}\theta

    Args:
        kc (ArrayLike): Complementary modulus. kc ∈ ℝ, kc ≠ 0.
        a, b (ArrayLike): Real-valued coefficient.

    Returns:
        Scalar or `numpy.ndarray` with the same shape as `kc`.

    Raises:
        ValueError: If kc = 0, more than one arguments are infinite, or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/cel2.html" width="100%" height="500px"></iframe>

    Special Cases:
        - cel2(kc, 0, 0) = 0
        - cel(kc, a, b) = 0 for \|kc\| = ∞
        - cel(kc, a, b) = sign(a) ∞ for \|a\| = ∞
        - cel(kc, a, b) = sign(b) ∞ for \|b\| = ∞

    Related Functions:
        - cel2(kc, a, b) = cel(kc, 1, a, b)
        - With kc² = 1 - m: E(m) = cel(kc, 1, 1, kc²) = cel2(kc, 1, kc²)

    References:
        - Bulirsch, Roland. “Numerical Calculation of Elliptic Integrals and Elliptic Functions.” Numerische Mathematik 7, no. 1 (February 1, 1965): 78-90. https://doi.org/10.1007/BF01397975.
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.cel2, kc, a, b)


def el1(x: ArrayLike, kc: ArrayLike) -> FloatArray | float:
    r"""Computes Bulirsch incomplete integral of the first kind ``el1``.

    .. math::

        \mathrm{el1}(x, k_c) = \int_0^{\arctan x} \frac{\mathrm{d}\theta}{\sqrt{\cos^2\theta + k_c^2\sin^2\theta}}

    Args:
        x (ArrayLike): Tangent of amplitude angle, x ∈ ℝ, x = tan(φ).
        kc (ArrayLike): Complementary modulus. kc ∈ ℝ, kc ≠ 0.

    Returns:
        Scalar or `numpy.ndarray` broadcast from inputs.

    Raises:
        ValueError: If kc = 0 or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/el1.html" width="100%" height="500px"></iframe>

    Special Cases:
        - el1(0, kc) = 0
        - el1(∞, kc) = cel1(kc)
        - el1(x, ∞) = 0

    Related Functions:
        With x = tan φ and kc² = 1 - m:
            - F(φ, m) = el1(x, kc) = el2(x, kc, 1, 1)
            - el1(∞, kc) = cel1(kc)

    References:
        - Bulirsch, Roland. “Numerical Calculation of Elliptic Integrals and Elliptic Functions.” Numerische Mathematik 7, no. 1 (February 1, 1965): 78-90. https://doi.org/10.1007/BF01397975.
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.el1, x, kc)


def el2(x: ArrayLike, kc: ArrayLike, a: ArrayLike, b: ArrayLike) -> FloatArray | float:
    r"""Computes Bulirsch incomplete integral of the second kind ``el2``.

    .. math::

        \mathrm{el2}(x, k_c, a, b) = \int_{0}^{\arctan x}\frac{a + b\,\tan^2\theta}{\sqrt{(1+\tan^2\theta)(1+k_c^2\tan^2\theta)}} \,\mathrm{d}\theta

    Args:
        x (ArrayLike): Tangent of amplitude angle, x ∈ ℝ, x = tan(φ).
        kc (ArrayLike): Complementary modulus. kc ∈ ℝ, kc ≠ 0.
        a, b (ArrayLike): Real-valued coefficient.

    Returns:
        Scalar or `numpy.ndarray` broadcast from inputs.

    Raises:
        ValueError: If kc = 0 or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/el2.html" width="100%" height="500px"></iframe>

    Special Cases:
        - el2(0, kc, a, b) = 0
        - el2(x, kc, 0, 0) = 0
        - el2(∞, kc, a, b) = cel2(kc, a, b)

    Related Functions:
        With x = tan φ and kc² = 1 - m,
            - F(φ, m) = el1(x, kc) = el2(x, kc, 1, 1)
            - E(φ, m) = el2(x, kc, 1, kc²)
            - el2(∞, kc, a, b) = cel2(kc, a, b)

    References:
        - Bulirsch, Roland. “Numerical Calculation of Elliptic Integrals and Elliptic Functions.” Numerische Mathematik 7, no. 1 (February 1, 1965): 78-90. https://doi.org/10.1007/BF01397975.
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.el2, x, kc, a, b)


def el3(x: ArrayLike, kc: ArrayLike, p: ArrayLike) -> FloatArray | float:
    r"""Computes Bulirsch incomplete integral of the third kind ``el3``.

    .. math::

        \mathrm{el3}(x, k_c, p) = \int_0^{\arctan x} \frac{\mathrm{d}\theta}{\left(\cos^2\theta + p\sin^2\theta\right)\,\sqrt{\cos^2\theta + k_c^2\sin^2\theta}}

    Args:
        x (ArrayLike): Tangent of amplitude angle, x ∈ ℝ, x = tan(φ).
        kc (ArrayLike): Complementary modulus. kc ∈ ℝ, kc ≠ 0.
        p (ArrayLike): Characteristic parameter. p ∈ ℝ, p ≠ 0.

    Returns:
        Scalar or `numpy.ndarray` broadcast from inputs. Returns the Cauchy principal value when 1 + px² < 0.

    Raises:
        ValueError: If kc = 0, 1 + px² = 0, \|kc\| > 1 for p < 0, or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/el3.html" width="100%" height="500px"></iframe>

    Special Cases:
        - el3(0, kc, p) = 0
        - el3(∞, kc, p) = cel(kc, p, 1, 1) = Π(1-p, 1-kc²)

    Related Functions:
        With x = tan φ, p = 1 - n and kc² = 1 - m:
            - Π(φ, n, m) = el3(x, kc, p)
            - el3(∞, kc, p) = cel(kc, p, 1, 1) = Π(n, m)

    References:
        - Bulirsch, R. “Numerical Calculation of Elliptic Integrals and Elliptic Functions. III.” Numerische Mathematik 13, no. 4 (August 1, 1969): 305-15. https://doi.org/10.1007/BF02165405.
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.el3, x, kc, p)



# EllipPy is licensed under The 3-Clause BSD, see LICENSE.
# Copyright 2025 Sira Pornsiriprasert <code@psira.me>

from __future__ import annotations
from importlib import metadata as _metadata

__all__ = ["__name__", "__version__", "__author__", "__license__", "__copyright__"]

__name__ = "ellippy"
__author__ = "Sira Pornsiriprasert"
__license__ = "BSD-3-Clause"
__copyright__ = f"2025 {__author__}"

try:
    __version__ = _metadata.version("ellippy")
except _metadata.PackageNotFoundError:
    # Fallback for editable installs or when metadata is unavailable
    __version__ = "0.0.0+unknown"



# EllipPy is licensed under The 3-Clause BSD, see LICENSE.
# Copyright 2025 Sira Pornsiriprasert <code@psira.me>

from __future__ import annotations

from .__about__ import *

from . import legendre, bulirsch, carlson, misc
from .legendre import *
from .bulirsch import *
from .carlson import *
from .misc import *

__all__ = [
    # Legendre complete
    "ellipk",
    "ellipe",
    "ellippi",
    "ellipd",
    # Legendre incomplete
    "ellipf",
    "ellipeinc",
    "ellippiinc",
    "ellipdinc",
    "ellippiinc_bulirsch",
    # Bulirsch
    "cel",
    "cel1",
    "cel2",
    "el1",
    "el2",
    "el3",
    # Carlson
    "elliprf",
    "elliprg",
    "elliprj",
    "elliprc",
    "elliprd",
    # Misc
    "jacobi_zeta",
    "heuman_lambda",
]



# EllipPy is licensed under The 3-Clause BSD, see LICENSE.
# Copyright 2025 Sira Pornsiriprasert <code@psira.me>

"""
Elliptic integral functions in Legendre's form.
"""

from numpy.typing import ArrayLike
from . import _ellip
from ._ellip import FloatArray, returnfloat, returnfloat_single


def ellipk(m: ArrayLike) -> FloatArray | float:
    r"""Computes complete elliptic integral of the first kind K(m).

    .. math::

        K(m) = \int_0^{\pi/2} \frac{\mathrm{d}\theta}{\sqrt{1 - m\,\sin^2\theta}}

    Args:
        m (ArrayLike): Elliptic parameter. m ∈ ℝ, m ≤ 1.

    Returns:
        Scalar or `numpy.ndarray` with the same shape as `m`.

    Raises:
        ValueError: If m > 1 or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/ellipk.html" width="100%" height="500px"></iframe>

    Special Cases:
        - K(0) = π/2
        - K(1) = ∞
        - K(-∞) = 0

    Related Functions:
        - K(m) = RF(0, 1 - m, 1).
        - F(π/2, m) = K(m).

    Notes:
        The elliptic modulus k is frequently used instead of the parameter m,
        where k² = m.

    References:
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Maddock, John, Paul Bristow, Hubert Holin, and Xiaogang Zhang. “Boost Math Library: Special Functions - Elliptic Integrals.” Accessed April 17, 2025. https://www.boost.org/doc/libs/1_88_0/libs/math/doc/html/math_toolkit/ellint.html.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat_single(_ellip.ellipk, m)


def ellipe(m: ArrayLike) -> FloatArray | float:
    r"""Computes complete elliptic integral of the second kind E(m).

    .. math::

        E(m) = \int_0^{\pi/2} \sqrt{1 - m\,\sin^2\theta}\,\mathrm{d}\theta

    Args:
        m (ArrayLike): Elliptic parameter. m ∈ ℝ, m ≤ 1.

    Returns:
        Scalar or `numpy.ndarray` with the same shape as `m`.

    Raises:
        ValueError: If m > 1 or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/ellipe.html" width="100%" height="500px"></iframe>

    Special Cases:
        - E(0) = π/2
        - E(1) = 1
        - E(-∞) = ∞

    Related Functions:
        - E(m) = 2·RG(0, 1 - m, 1).
        - E(π/2, m) = E(m).

    Notes:
        The elliptic modulus k is frequently used instead of the parameter m,
        where k² = m.

    References:
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Maddock, John, Paul Bristow, Hubert Holin, and Xiaogang Zhang. “Boost Math Library: Special Functions - Elliptic Integrals.” Accessed April 17, 2025. https://www.boost.org/doc/libs/1_88_0/libs/math/doc/html/math_toolkit/ellint.html.
        - Abramowitz, Milton, and Irene A. Stegun. Handbook of Mathematical Functions: With Formulas, Graphs and Mathematical Tables. Unabridged, Unaltered and corr. Republ. of the 1964 ed. With Conference on mathematical tables, National science foundation, and Massachusetts institute of technology. Dover Books on Advanced Mathematics. Dover publ, 1972.
        - The SciPy community. “Scipy.Special.Ellipe — SciPy v1.16.0 Manual.” Accessed July 28, 2025. https://docs.scipy.org/doc/scipy-1.16.0/reference/generated/scipy.special.ellipe.html.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat_single(_ellip.ellipe, m)


def ellippi(n: ArrayLike, m: ArrayLike) -> FloatArray | float:
    r"""Computes complete elliptic integral of the third kind Π(n | m).

    .. math::

        \Pi(n\,|\,m) = \int_0^{\pi/2} \frac{\mathrm{d}\theta}{\left(1 - n\,\sin^2\theta\right)\,\sqrt{1 - m\,\sin^2\theta}}

    Args:
        n (ArrayLike): Characteristic. n ∈ ℝ, n ≠ 1.
        m (ArrayLike): Elliptic parameter. m ∈ ℝ, m ≤ 1.

    Returns:
        Scalar or `numpy.ndarray` broadcast from `n` and `m`. Returns the Cauchy principal value if n > 1.

    Raises:
        ValueError: If n = 1, m > 1, or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/ellippi_3d.html" width="100%" height="650px"></iframe>

    Special Cases:
        - Π(0, 0) = π/2
        - Π(0, m) = K(m)
        - Π(n, 0) = π/(2\sqrt{1-n}) for n < 1
        - Π(n, 0) = 0 for n > 1
        - Π(n, m) = ∞ as n → 1-
        - Π(n, 1) = sign(1-n) · ∞
        - Π(±∞, m) = 0
        - Π(n, -∞) = 0

    Related Functions:
        - Π(n, m) = (n/3)·RJ(0, 1 - m, 1, 1 - n) + K(m)
        - Π(n, n) = E(n)/(1 - n) for n < 1
        - Π(n, m) = K(m) - E(m)/(1 - m) for n → 1+

    Notes:
        - The elliptic modulus k is frequently used instead of the parameter m, where k² = m.
        - The characteristic n is sometimes expressed in term of α, where α² = n.

    References:
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Maddock, John, Paul Bristow, Hubert Holin, and Xiaogang Zhang. “Boost Math Library: Special Functions - Elliptic Integrals.” Accessed April 17, 2025. https://www.boost.org/doc/libs/1_88_0/libs/math/doc/html/math_toolkit/ellint.html.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.ellippi, n, m)


def ellipd(m: ArrayLike) -> FloatArray | float:
    r"""Computes complete elliptic integral of Legendre's type D(m).

    .. math::

        D(m) = \int_0^{\pi/2} \frac{\sin^2\theta}{\sqrt{1 - m\,\sin^2\theta}}\,\mathrm{d}\theta

    Args:
        m (ArrayLike): Elliptic parameter. m ∈ ℝ, m ≤ 1.

    Returns:
        Scalar or `numpy.ndarray` with the same shape as `m`.

    Raises:
        ValueError: If m > 1 or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/ellipd.html" width="100%" height="500px"></iframe>

    Special Cases:
        - D(0) = π/4
        - D(1) = ∞
        - D(-∞) = 0

    Related Functions:
        - D(m) = (K(m) - E(m)) / m
        - D(m) = RD(0, 1 - m, 1) / 3
        - D(π/2, m) = D(m)

    Notes:
        The elliptic modulus k is frequently used instead of m, where k² = m.

    References:
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Maddock, John, Paul Bristow, Hubert Holin, and Xiaogang Zhang. “Boost Math Library: Special Functions - Elliptic Integrals.” Accessed April 17, 2025. https://www.boost.org/doc/libs/1_88_0/libs/math/doc/html/math_toolkit/ellint.html.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat_single(_ellip.ellipd, m)


def ellipf(phi: ArrayLike, m: ArrayLike) -> FloatArray | float:
    r"""Computes incomplete elliptic integral of the first kind F(φ | m).

    .. math::

        F(\varphi\,|\,m) = \int_0^{\varphi} \frac{\mathrm{d}\theta}{\sqrt{1 - m\,\sin^2\theta}}

    Args:
        phi (ArrayLike): Amplitude angle (φ) in radians. φ ∈ ℝ. 
        m (ArrayLike): Elliptic parameter. m ∈ ℝ.

    Returns:
        Scalar or `numpy.ndarray` broadcast from `phi` and `m`.

    Raises:
        ValueError: If m sin²φ > 1 or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/ellipf.html" width="100%" height="500px"></iframe>

    Special Cases:
        - F(0, m) = 0
        - F(φ, 0) = φ
        - F(π/2, m) = K(m)
        - F(φ, -∞) = 0
        - F(φ, m) = sign(φ) ∞ for \|φ\| = ∞

    Related Functions:
        - With c = csc²φ: F(φ, m) = RF(c - 1, c - m, c)

    Notes:
        The elliptic modulus k is frequently used instead of the parameter m, where k² = m.

    References:
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Maddock, John, Paul Bristow, Hubert Holin, and Xiaogang Zhang. “Boost Math Library: Special Functions - Elliptic Integrals.” Accessed April 17, 2025. https://www.boost.org/doc/libs/1_88_0/libs/math/doc/html/math_toolkit/ellint.html.
        - The MathWorks, Inc. “ellipticF.” Accessed April 21, 2025. https://www.mathworks.com/help/symbolic/sym.ellipticf.html.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.ellipf, phi, m)


def ellipeinc(phi: ArrayLike, m: ArrayLike) -> FloatArray | float:
    r"""Computes incomplete elliptic integral of the second kind E(φ | m).

    .. math::

        E(\varphi\,|\,m) = \int_0^{\varphi} \sqrt{1 - m\,\sin^2\theta}\,\mathrm{d}\theta

    Args:
        phi (ArrayLike): Amplitude angle (φ) in radians. φ ∈ ℝ. 
        m (ArrayLike): Elliptic parameter. m ∈ ℝ.

    Returns:
        Scalar or `numpy.ndarray` broadcast from `phi` and `m`.

    Raises:
        ValueError: If m sin²φ > 1 or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/ellipeinc.html" width="100%" height="500px"></iframe>

    Special Cases:
        - E(0, m) = 0
        - E(φ, 0) = φ
        - E(π/2, m) = E(m)
        - E(φ, 1) = sin(φ)
        - E(φ, -∞) = ∞
        - E(±∞, m) = ±∞

    Related Functions:
        - With c = csc²φ: E(φ, m) = RF(c - 1, c - m, c) - m/3 · RD(c - 1, c - m, c)

    Notes:
        The elliptic modulus k is frequently used instead of the parameter m, where k² = m.

    References:
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Maddock, John, Paul Bristow, Hubert Holin, and Xiaogang Zhang. “Boost Math Library: Special Functions - Elliptic Integrals.” Accessed April 17, 2025. https://www.boost.org/doc/libs/1_88_0/libs/math/doc/html/math_toolkit/ellint.html.
        - The MathWorks, Inc. “ellipticE.” Accessed April 21, 2025. https://www.mathworks.com/help/symbolic/sym.elliptice.html.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.ellipeinc, phi, m)


def ellippiinc(n: ArrayLike, phi: ArrayLike, m: ArrayLike) -> FloatArray | float:
    r"""Computes incomplete elliptic integral of the third kind Π(n; φ | m).

    .. math::

        \Pi(n;\,\varphi\,|\,m) = \int_0^{\varphi} \frac{\mathrm{d}\theta}{\left(1 - n\,\sin^2\theta\right)\,\sqrt{1 - m\,\sin^2\theta}}

    Args:
        n (ArrayLike): Characteristic. n ∈ ℝ, n ≠ 1.
        phi (ArrayLike): Amplitude angle (φ) in radians. φ ∈ ℝ. 
        m (ArrayLike): Elliptic parameter. m ∈ ℝ.

    Returns:
        Scalar or `numpy.ndarray` broadcast from `n`, `phi`, and `m`. Returns the Cauchy principal value if n sin²φ > 1.

    Raises:
        ValueError: If m sin²φ > 1, n sin²φ = 1, m ≥ 1 with φ not a multiple of π/2, or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/ellippiinc_3d.html" width="100%" height="650px"></iframe>

    Special Cases:
        - Π(0, n, m) = 0
        - Π(φ, 0, 0) = φ
        - Π(φ, 1, 0) = tan(φ)
        - Π(φ, 0, m) = F(φ, m)

    Related Functions:
        - Π(φ, 0, m) = F(φ, m)
        - With c = csc²φ: Π(φ, n, m) = (n/3) · RJ(c - 1, c - m, c, c - n) + F(φ, m)
        - With x = tan φ, p = 1 - n, and k_c² = 1 - m: Π(φ, n, m) = el3(x, k_c, p)

    Notes:
        - The elliptic modulus k is frequently used instead of m, where k² = m.
        - The characteristic n is sometimes expressed in term of α, where α² = n.
        - This function `ellippiinc` (of the package) is the circular or hyperbolic case of Π,
          since n and m are real. It is called circular if n·(n - m)·(n - 1) is negative and
          hyperbolic if it is positive.

    References:
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Maddock, John, Paul Bristow, Hubert Holin, and Xiaogang Zhang. “Boost Math Library: Special Functions - Elliptic Integrals.” Accessed April 17, 2025. https://www.boost.org/doc/libs/1_88_0/libs/math/doc/html/math_toolkit/ellint.html.
        - Wolfram Research. “EllipticPi.” 2022. https://reference.wolfram.com/language/ref/EllipticPi.html.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.ellippiinc, n, phi, m)


def ellipdinc(phi: ArrayLike, m: ArrayLike) -> FloatArray | float:
    r"""Computes incomplete elliptic integral of Legendre's type D(φ | m).

    .. math::

        D(\varphi\,|\,m) = \int_0^{\varphi} \frac{\sin^2\theta}{\sqrt{1 - m\,\sin^2\theta}}\,\mathrm{d}\theta

    Args:
        phi (ArrayLike): Amplitude angle (φ) in radians. φ ∈ ℝ. 
        m (ArrayLike): Elliptic parameter. m ∈ ℝ.

    Returns:
        Scalar or `numpy.ndarray` broadcast from `phi` and `m`.

    Raises:
        ValueError: If m sin²φ > 1 or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/ellipdinc.html" width="100%" height="500px"></iframe>

    Special Cases:
        - D(0, m) = 0
        - D(π/2, m) = D(m)
        - D(φ, -∞) = 0
        - D(±∞, m) = ±∞

    Related Functions:
        With c = csc²φ,
            - D(φ, m) = (F(φ, m) - E(φ, m)) / m
            - D(φ, m) = RD(c - 1, c - m, c) / 3

    Notes:
        The elliptic modulus k is frequently used instead of m, where k² = m.

    References:
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Maddock, John, Paul Bristow, Hubert Holin, and Xiaogang Zhang. “Boost Math Library: Special Functions - Elliptic Integrals.” Accessed April 17, 2025. https://www.boost.org/doc/libs/1_88_0/libs/math/doc/html/math_toolkit/ellint.html.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.ellipdinc, phi, m)


def ellippiinc_bulirsch(
    n: ArrayLike, phi: ArrayLike, m: ArrayLike
) -> FloatArray | float:
    r"""Computes incomplete elliptic integral of the third kind using the Bulirsch algorithm.

    This is typically about 2x as fast as the standard implementation for non-PV cases
    and m < 1 cases, while maintaining similar accuracy. Otherwise, it delegates
    to :func:`ellippiinc`.

    Args:
        n (ArrayLike): Characteristic. n ∈ ℝ, n ≠ 1.
        phi (ArrayLike): Amplitude angle (φ) in radians. φ ∈ ℝ. 
        m (ArrayLike): Elliptic parameter. m ∈ ℝ.

    Returns:
        Scalar or `numpy.ndarray` broadcast from `n`, `phi`, and `m`.

    Notes:
        See :func:`ellippiinc` for definitions, domains, and relationships.

    References:
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.ellippiinc_bulirsch, n, phi, m)



# EllipPy is licensed under The 3-Clause BSD, see LICENSE.
# Copyright 2025 Sira Pornsiriprasert <code@psira.me>

"""
Elliptic integral functions in Carlson's form.
"""

from numpy.typing import ArrayLike
from . import _ellip
from ._ellip import FloatArray, returnfloat


def elliprf(x: ArrayLike, y: ArrayLike, z: ArrayLike) -> FloatArray | float:
    r"""Computes Carlson symmetric integral RF.

    .. math::

        R_F(x, y, z) = \tfrac{1}{2} \int_0^{\infty} \frac{\mathrm{d}t}{\sqrt{(t+x)(t+y)(t+z)}}

    Args:
        x, y, z (ArrayLike): Real-valued parameter.

    Returns:
        Scalar or `numpy.ndarray` broadcast from inputs.

    Raises:
        ValueError: If any of x, y, or z is negative, or more than one of them are zero, or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/elliprf.html" width="100%" height="500px"></iframe>

    Special Cases:
        - RF(x, x, x) = 1/sqrt(x)
        - RF(x, y, y) = RC(x, y)
        - RF(0, y, y) = π/(2 sqrt(y))
        - RF(x, y, z) = 0 for x = ∞ or y = ∞ or z = ∞

    Related Functions:
        With c = csc²φ, r = 1/x², and kc² = 1 - m,
        - F(φ,m) = RF(c - 1, c - m, c)
        - el1(x, kc) = RF(r, r + m, r + 1)

    Notes:
        The parameters x, y, and z are symmetric. This means swapping them does not change the 
        value of the function. At most one of them can be zero.
    
    References:
        - Maddock, John, Paul Bristow, Hubert Holin, and Xiaogang Zhang. “Boost Math Library: Special Functions - Elliptic Integrals.” Accessed April 17, 2025. https://www.boost.org/doc/libs/1_88_0/libs/math/doc/html/math_toolkit/ellint.html.
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.elliprf, x, y, z)


def elliprg(x: ArrayLike, y: ArrayLike, z: ArrayLike) -> FloatArray | float:
    r"""Computes Carlson symmetric integral RG.

    .. math::

        R_G(x,y,z) = \frac{1}{4}\int_{0}^{\infty}\frac{t}
            {\sqrt{(t+x)(t+y)(t+z)}}\,\left(\dfrac{x}{t+x}+\dfrac{y}{t+y}+\dfrac{z}{t+z}\right)\,\mathrm{d}t
    Args:
        x, y, z (ArrayLike): Real-valued parameter.

    Returns:
        Scalar or `numpy.ndarray` broadcast from inputs.
    
    Raises:
        ValueError: If any of x, y, or z is negative or infinite, or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/elliprg.html" width="100%" height="500px"></iframe>
    
    Special Cases:
        - RG(x, x, x) = sqrt(x)
        - RG(0, y, y) = π/4 * sqrt(y)
        - RG(x, y, y) = (y * RC(x, y) + sqrt(x))/2
        - RG(0, 0, z) = sqrt(z)/2

    Related Functions:
        With c = csc²φ, r = 1/x², and kc² = 1 - m,
            - E(m) = 2·RG(0, kc², 1)
            - 5(φ, m) = 2·RG(c - 1, c - m, c) - (c - 1)·RF(c - 1, c - m, c) - sqrt((c - 1) * (c - m) / c)

    Notes:
        The parameters x, y, and z are symmetric. This means swapping them does not change the 
        value of the function. At most one of them can be zero.
    
    References:
        - Maddock, John, Paul Bristow, Hubert Holin, and Xiaogang Zhang. “Boost Math Library: Special Functions - Elliptic Integrals.” Accessed April 17, 2025. https://www.boost.org/doc/libs/1_88_0/libs/math/doc/html/math_toolkit/ellint.html.
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.elliprg, x, y, z)


def elliprj(
    x: ArrayLike, y: ArrayLike, z: ArrayLike, p: ArrayLike
) -> FloatArray | float:
    r"""Computes Carlson symmetric integral RJ.

    .. math::

        R_J(x, y, z, p) = \tfrac{3}{2} \int_0^{\infty} \frac{\mathrm{d}t}{(t+p)\,\sqrt{(t+x)(t+y)(t+z)}}
    
    Args:
        x, y, z (ArrayLike): Real-valued parameter.
        p (ArrayLike): Real-valued parameter. p ∈ ℝ, p ≠ 0.

    Returns:
        Scalar or `numpy.ndarray` broadcast from inputs. Returns the Cauchy principal value if p < 0.

    Raises:
        ValueError: If any of x, y, or z is negative, or more than one of them are zero, or p = 0, or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/elliprj.html" width="100%" height="500px"></iframe>
    
    Special Cases:
        - RJ(x, x, x, x) = 1/(x sqrt(x))
        - RJ(x, y, z, z) = RD(x, y, z)
        - RJ(x, x, x, p) = 3/(x-p) * (RC(x, p) - 1/sqrt(x)) for x ≠ p and xp ≠ 0
        - RJ(x, y, y, y) = RD(x, y, y)
        - RJ(x, y, y, p) = 3/(p-y) * (RC(x, y) - RC(x, p)) for y ≠ p
        - RJ(x, y, z, p) = 0 for x = ∞ or y = ∞ or z = ∞ or p = ∞

    Related Functions:
        - With c = csc²φ and kc² = 1 - m: Π(φ, n, m) = n / 3 * RJ(c - 1, c - m, c, c - n) + F(φ, m)

    Notes:
        The parameters x, y, and z are symmetric. This means swapping them does not change the 
        value of the function. At most one of them can be zero.
    
    References:
        - Maddock, John, Paul Bristow, Hubert Holin, and Xiaogang Zhang. “Boost Math Library: Special Functions - Elliptic Integrals.” Accessed April 17, 2025. https://www.boost.org/doc/libs/1_88_0/libs/math/doc/html/math_toolkit/ellint.html.
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.elliprj, x, y, z, p)


def elliprc(x: ArrayLike, y: ArrayLike) -> FloatArray | float:
    r"""Computes Carlson degenerate integral RC.

    .. math::

        R_C(x, y) = \tfrac{1}{2} \int_0^{\infty} \frac{\mathrm{d}t}{(t+y)\,\sqrt{t+x}}
    
    Args:
        x, y (ArrayLike): Real-valued parameter.

    Returns:
        Scalar or `numpy.ndarray` broadcast from inputs. Returns the Cauchy principal value if y < 0.

    Raises:
        ValueError: If x < 0 or y = 0, or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/elliprc.html" width="100%" height="500px"></iframe>
    
    Special Cases:
        - RC(x, x) = 1/sqrt(x)
        - RC(0, y) = π/(2*sqrt(y))
        - RC(x, y) = atan(sqrt(y-x)/x) / sqrt(y-x) for y > x
        - RC(x, y) = ln(sqrt(x) + sqrt(x-y)) / sqrt(x-y) for y < x
        - RC(x, y) = 0 for x = ∞ or y = ∞

    Related Functions:
        - RC(x, y) = RF(x, y, y)

    Notes:
        RC is a degenerate case of the RF. It is an elementary function rather than an elliptic integral.
    
    References:
        - Maddock, John, Paul Bristow, Hubert Holin, and Xiaogang Zhang. “Boost Math Library: Special Functions - Elliptic Integrals.” Accessed April 17, 2025. https://www.boost.org/doc/libs/1_88_0/libs/math/doc/html/math_toolkit/ellint.html.
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - The SciPy Community. “SciPy: Special Functions - Elliprc.” Accessed April 17, 2025. https://docs.scipy.org/doc/scipy/reference/generated/scipy.special.elliprc.html.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.elliprc, x, y)


def elliprd(x: ArrayLike, y: ArrayLike, z: ArrayLike) -> FloatArray | float:
    r"""Computes Carlson degenerate integral RD.

    .. math::

        R_D(x, y, z) = \tfrac{3}{2} \int_0^{\infty} \frac{\mathrm{d}t}{(t+z)\,\sqrt{(t+x)(t+y)(t+z)}}
    
    Args:
        x, y, z (ArrayLike): Real-valued parameter.

    Returns:
        Scalar or `numpy.ndarray` broadcast from inputs.

    Raises:
        ValueError: If x < 0, y < 0, z ≤ 0 or when both x and y are zero, or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/elliprd.html" width="100%" height="500px"></iframe>
    
    Special Cases:
        - RD(x, x, x) = 1/(x sqrt(x))
        - RD(0, y, y) = 3/4 * π / (y sqrt(y))
        - RD(x, y, y) = 3/(2(y-x)) * (RC(x, y) - sqrt(x)/y) for x ≠ y
        - RD(x, x, z) = 3/(z-x) * (RC(z, x) - 1/sqrt(z)) for x ≠ z
        - RD(x, y, z) = 0 for x = ∞ or y = ∞ or z = ∞

    Related Functions:
        - With c = csc²φ: D(φ, m) = RD(c - 1, c - m, c) / 3

    Notes:
        The parameters x and y (but not z!) are symmetric. This means swapping them does not change the value of the function. At most one of them can be zero.
    
    References:
        - Maddock, John, Paul Bristow, Hubert Holin, and Xiaogang Zhang. “Boost Math Library: Special Functions - Elliptic Integrals.” Accessed April 17, 2025. https://www.boost.org/doc/libs/1_88_0/libs/math/doc/html/math_toolkit/ellint.html.
        - Carlson, B. C. “DLMF: Chapter 19 Elliptic Integrals.” Accessed February 19, 2025. https://dlmf.nist.gov/19.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.elliprd, x, y, z) 


# EllipPy is licensed under The 3-Clause BSD, see LICENSE.
# Copyright 2025 Sira Pornsiriprasert <code@psira.me>

from collections.abc import Callable
import numpy as np
from numpy.typing import ArrayLike, NDArray

from .ellippy_binding import (
    ellipk,
    ellipe,
    ellippi,
    ellipd,
    ellipf,
    ellipeinc,
    ellippiinc,
    ellipdinc,
    ellippiinc_bulirsch,
    cel,
    cel1,
    cel2,
    el1,
    el2,
    el3,
    elliprf,
    elliprg,
    elliprj,
    elliprc,
    elliprd,
    jacobi_zeta,
    heuman_lambda,
)

FloatArray = NDArray[np.float64]


def asarray(x: ArrayLike) -> FloatArray:
    return np.array(x, dtype=np.float64).flatten()


def returnfloat_single(func: Callable, arg: ArrayLike) -> FloatArray | float:
    try:
        ans = func(asarray(arg))
    except RuntimeError as e:
        raise ValueError(e)
    return ans.item() if isinstance(arg, float) else ans


def returnfloat(func: Callable, *args: ArrayLike) -> FloatArray | float:
    args_asarray = tuple(asarray(arg) for arg in args)
    try:
        ans = func(*args_asarray)
    except RuntimeError as e:
        raise ValueError(e)
    return ans.item() if isinstance(args[0], float) else ans



# EllipPy is licensed under The 3-Clause BSD, see LICENSE.
# Copyright 2025 Sira Pornsiriprasert <code@psira.me>

"""
Miscellaneous functions related to elliptic integrals.
"""

from numpy.typing import ArrayLike
from . import _ellip
from ._ellip import FloatArray, returnfloat


def jacobi_zeta(phi: ArrayLike, m: ArrayLike) -> FloatArray | float:
    r"""Computes Jacobi Zeta function Z(φ | m).

    .. math::

        Z(\varphi, m) = E(\varphi, m) - \frac{E(m)\,F(\varphi, m)}{K(m)}

    Args:
        phi (ArrayLike): Amplitude angle (φ) in radians. φ ∈ ℝ. 
        m (ArrayLike): Elliptic parameter. m ∈ ℝ, m ≤ 1.

    Returns:
        Scalar or `numpy.ndarray` broadcast from inputs.

    Raises:
        ValueError: If any m > 1, or phi/m are infinite, or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/jacobi_zeta.html" width="100%" height="500px"></iframe>

    Special Cases:
        - Z(0, m) = 0
        - Z(φ, 0) = 0
        - Z(φ, 1) = sin(φ)·sign(cos(φ)) for φ ≠ nπ/2
        - Z(nπ/2, m) = 0 for n ∈ ℤ
        - Z(φ, m) = -Z(−φ, m)

    Related Functions:
        - Z(φ, m) = E(φ, m) - E(m) F(φ, m) / K(m)
        - Z(φ, m) = m sin(φ) cos(φ) √(1 - m sin²φ) · RJ(0, k_c², 1, 1 - m sin²φ) / (3 K(m))

    References:
        - Maddock, John, Paul Bristow, Hubert Holin, and Xiaogang Zhang. “Boost Math Library: Special Functions - Elliptic Integrals.” Accessed August 30, 2025. https://www.boost.org/doc/libs/1_88_0/libs/math/doc/html/math_toolkit/ellint.html.
        - Reinhardt, W. P., and P. L. Walker. “DLMF: Chapter 22 Jacobian Elliptic Functions.” Accessed August 31, 2025. https://dlmf.nist.gov/22.
        - Weisstein, Eric W. “Jacobi Zeta Function.” Wolfram Research, Inc. Accessed August 31, 2025. https://mathworld.wolfram.com/JacobiZetaFunction.html.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.jacobi_zeta, phi, m)


def heuman_lambda(phi: ArrayLike, m: ArrayLike) -> FloatArray | float:
    r"""Computes Heuman Lambda function Λ₀(φ | m).

    .. math::

        \Lambda_0(\varphi, m) = \frac{F\!\left(\varphi, 1-m\right)}{K\!\left(1-m\right)} + \frac{2}{\pi} K(m)\, Z\!\left(\varphi, 1-m\right)

    Args:
        phi (ArrayLike): Amplitude angle (φ) in radians. φ ∈ ℝ. 
        m (ArrayLike): Elliptic parameter. m ∈ ℝ, 0 ≤ m < 1.

    Returns:
        Scalar or `numpy.ndarray` broadcast from inputs.

    Raises:
        ValueError: If m < 0 or m ≥ 1, phi is infinite, or inputs contain NaN.

    Graph:
        .. raw:: html

            <iframe src="./../_static/figures/heuman_lambda.html" width="100%" height="500px"></iframe>

    Special Cases:
        - Λ₀(nπ/2, m) = n where n ∈ ℤ
        - Λ0(φ, 0) = sin(φ)

    Related Functions:
        With mc = 1 - m and Δ² = 1 - mc sin²φ:
            - Λ₀(φ, m) = F(φ, mc)/K(mc) + (2/π) K(m) Z(φ, mc)
            - Λ₀(φ, m) = 2/π · mc sin(φ) cos(φ)/Δ · [RF(0, mc, 1) + m/(3Δ²) RJ(0, mc, 1, 1 - m/Δ²)]

    References:
        - Maddock, John, Paul Bristow, Hubert Holin, and Xiaogang Zhang. “Boost Math Library: Special Functions - Elliptic Integrals.” Accessed August 30, 2025. https://www.boost.org/doc/libs/1_88_0/libs/math/doc/html/math_toolkit/ellint.html.
        - Pornsiriprasert, Sira. Ellip: Elliptic Integrals for Rust. V. 0.5.1. Released October 10, 2025. https://docs.rs/ellip/0.5.1/ellip/index.html.
    """
    return returnfloat(_ellip.heuman_lambda, phi, m)
